(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     60786,       1846]
NotebookOptionsPosition[     46008,       1545]
NotebookOutlinePosition[     48470,       1607]
CellTagsIndexPosition[     48384,       1602]
WindowTitle->CakeWaveletStack
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["LIE ANALYSIS SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox[
        InterpretationBox[Cell[TextData[{
          "Tutorials",
          StyleBox[" \[FilledDownTriangle]", "AnchorBarArrow",
           StripOnInput->False]
         }]],
         TextCell[
          Row[{"Tutorials", 
            Style[" \[FilledDownTriangle]", "AnchorBarArrow"]}]]],
        StripOnInput->False], {"\<\"Basic Usage\"\>":>
       Documentation`HelpLookup["paclet:LieAnalysis/tutorial/BasicUsage"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"Tutorials"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox[
        InterpretationBox[Cell[TextData[{
          "See Also",
          StyleBox[" \[FilledDownTriangle]", "AnchorBarArrow",
           StripOnInput->False]
         }]],
         TextCell[
          Row[{"See Also", 
            Style[" \[FilledDownTriangle]", "AnchorBarArrow"]}]]],
        StripOnInput->False], {"\<\"GaborWaveletStack\"\>":>
       Documentation`HelpLookup[
       "paclet:LieAnalysis/ref/GaborWaveletStack"], "\<\"CakeWaveletStack3D\"\
\>":>Documentation`HelpLookup[
       "paclet:LieAnalysis/ref/CakeWaveletStack3D"], \
"\<\"OrientationScoreTransform\"\>":>Documentation`HelpLookup[
       "paclet:LieAnalysis/ref/OrientationScoreTransform"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"SeeAlso"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox[
        InterpretationBox[Cell[TextData[{
          "Related Guides",
          StyleBox[" \[FilledDownTriangle]", "AnchorBarArrow",
           StripOnInput->False]
         }]],
         TextCell[
          Row[{"Related Guides", 
            Style[" \[FilledDownTriangle]", "AnchorBarArrow"]}]]],
        StripOnInput->False], {"\<\"LieAnalysis Package\"\>":>
       Documentation`HelpLookup["paclet:LieAnalysis/guide/LieAnalysis"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"MoreAbout"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox[
        InterpretationBox[Cell[TextData[{
          "URL",
          StyleBox[" \[FilledDownTriangle]", "AnchorBarArrow",
           StripOnInput->False]
         }]],
         TextCell[
          Row[{"URL", 
            Style[" \[FilledDownTriangle]", "AnchorBarArrow"]}]]],
        StripOnInput->False], {"\<\"LieAnalysis/ref/CakeWaveletStack\"\>":>
       None, "\<\"Copy Wolfram Documentation Center URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["LieAnalysis/ref/CakeWaveletStack"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/language/LieAnalysis/ref/\
CakeWaveletStack.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
             If[
              TrueQ[DocumentationBuild`Make`Private`wsmlinkQ$10095], 
              "http://reference.wolfram.com/system-modeler/", 
              "http://reference.wolfram.com/language/"], 
             "LieAnalysis/ref/CakeWaveletStack", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"URLMenu"]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[TextData[{
 Cell["CakeWaveletStack", "ObjectName"],
 Cell[BoxData[
  TemplateBox[{8},
   "Spacer1"]]],
 Cell[BoxData[""], "ObjectNameTranslation"]
}], "ObjectNameGrid"],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"CakeWaveletStack", "[", 
       StyleBox[
        RowBox[{"n", ",", " ", "size"}], "TI"], "]"}]], "InlineFormula"],
     " \[LineSeparator]constructs a Cake Wavelet stack having ",
     StyleBox["n", "TI"],
     " orientations, dimensions ",
     StyleBox["size", "TI"],
     " x ",
     StyleBox["size", "TI"],
     " with default parameters."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"CakeWaveletStack", "[", 
       RowBox[{
        StyleBox["n", "TI"], ",", 
        StyleBox["size", "TI"], ",", " ", 
        StyleBox["options", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]constructs a Cake Wavelet stack having ",
     StyleBox["n", "TI"],
     " orientations, dimensions ",
     StyleBox["size", "TI"],
     " x ",
     StyleBox["size", "TI"],
     " with the specified ",
     StyleBox["options", "TI"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {None, {None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[Cell[TextData[{
   Cell[BoxData[
    TemplateBox[{6},
     "Spacer1"]]],
   "Details"
  }], "NotesFrameText"],
  Appearance->{Automatic, None, "Normal", Automatic},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->1],

Cell["", "SectionHeaderSpacer"],

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["CakeWaveletStack"]],"paclet:LieAnalysis/ref/CakeWaveletStack"},
   
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 " creates a set of (complex) orientation sensitive wavelets that can be used \
to create an Orientation Score from a 2D image (see ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["OrientationScoreTransform"]],
    "paclet:LieAnalysis/ref/OrientationScoreTransform"},
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 ")."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "Cake wavelets ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    SuperscriptBox[
     OverscriptBox["\[Psi]", "^"], "cake"], ")"}], TraditionalForm]]],
 " are constructed in the Fourier domain as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      OverscriptBox["\[Psi]", "^"], "cake"], "(", 
     StyleBox["\[Omega]",
      FontWeight->"Bold"], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["B", "k"], "(", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"\[CurlyPhi]", " ", "mod", " ", "2", "\[Pi]"}], ")"}], " ", 
        "-", " ", 
        RowBox[{"\[Pi]", "/", "2"}]}], 
       SubscriptBox["s", "\[Theta]"]], ")"}], " ", 
     RowBox[{
      SubscriptBox["\[ScriptCapitalM]", "N"], "(", "\[Rho]", ")"}]}]}], 
   TraditionalForm]]],
 ", with ",
 StyleBox["\[Omega] ",
  FontWeight->"Bold"],
 "= (\[Rho] cos \[CurlyPhi], \[Rho] sin \[CurlyPhi]). ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "k"], TraditionalForm]]],
 " denotes the ",
 Cell[BoxData[
  SuperscriptBox["k", "th"]], "InlineFormula"],
 " order B-spline given by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["B", "k"], 
     RowBox[{"(", "x", ")"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["B", 
        RowBox[{"k", "-", "1"}]], " ", "*", " ", 
       SubscriptBox["B", "0"]}], ")"}], 
     RowBox[{"(", "x", ")"}]}]}], TraditionalForm]]],
 ", with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["B", "0"], "=", 
    TagBox[GridBox[{
       {"\[Piecewise]", GridBox[{
          {"1", 
           RowBox[{
            RowBox[{"if", " ", "-", 
             RowBox[{"1", "/", "2"}]}], " ", "<", " ", "x", " ", "<", " ", 
            RowBox[{
             RowBox[{"+", "1"}], "/", "2"}]}]},
          {"0", "otherwise"}
         },
         AllowedDimensions->{2, Automatic},
         Editable->True,
         GridBoxAlignment->{
          "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
         GridBoxItemSize->{
          "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, 
           "Rows" -> {{1.}}, "RowsIndexed" -> {}},
         GridBoxSpacings->{"Columns" -> {
             Offset[0.27999999999999997`], {
              Offset[0.84]}, 
             Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
             Offset[0.2], {
              Offset[0.4]}, 
             Offset[0.2]}, "RowsIndexed" -> {}},
         Selectable->True]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{
       "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.35]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}],
     "Piecewise",
     DeleteWithContents->True,
     Editable->False,
     SelectWithContents->True,
     Selectable->False]}], TraditionalForm]]],
 ".\[LineSeparator]",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[ScriptCapitalM]", "N"], "(", "\[Rho]", ")"}], 
   TraditionalForm]]],
 " specifies the radial function given by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalM]", "N"], "(", "\[Rho]", ")"}], " ", "=", 
    " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        SuperscriptBox["\[Rho]", "2"], "t"]], 
      RowBox[{
       StyleBox[
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"k", "=", "0"}], "N"],
        FontSize->22], 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[Rho]", "2"], 
           SuperscriptBox["t", 
            RowBox[{"-", "1"}]]}], ")"}], "k"], 
        RowBox[{"k", "!"}]]}]}], "\[LessEqual]", " ", "1."}]}], 
   TraditionalForm]]],
 " "
}], "Notes",
 CellID->233796496],

Cell[TextData[{
 "Applying ",
 ButtonBox["Normal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Normal"],
 " to the output of ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["CakeWaveletStack"]],"paclet:LieAnalysis/ref/CakeWaveletStack"},
   
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 " returns an ",
 ButtonBox["Association.",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Association"]
}], "Notes",
 CellID->359789379],

Cell[TextData[{
 "The output of ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["CakeWaveletStack"]],"paclet:LieAnalysis/ref/CakeWaveletStack"},
   
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 " is an ObjCakeWavelet, which is a child of the ",
 StyleBox["ObjOrientationWavelet",
  FontSlant->"Italic"],
 "."
}], "Notes",
 CellID->353028093],

Cell["The following options can be given:", "Notes",
 CellID->507720613],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], "\"\<Directional\>\"", 
    ButtonBox["False",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/False"], Cell[
    "Whenever the spatial wavelets are single or double headed", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<Design\>\"", "\"\<N\>\"", Cell[
    "Specifies the design constraints for the wavelets", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<InflectionPoint\>\"", "0.8", Cell["\<\
Specifies the inflection point in radial direction of the gaussian window\
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<SplineOrder\>\"", "3", Cell["\<\
Specifies the order of the B-Spline that is used to create the wavelets\
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<MnOrder\>\"", "8", Cell[
    "Specifies the order of the MnWindow", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<DcStandardDeviation\>\"", "8", 
    Cell["\<\
The standard deviation of the gaussian that is used to remove the center\
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<OverlapFactor\>\"", "1", Cell["\<\
Sets how many B-Splines overlap, thereby not narrowing the angular resolution\
\
\>", "TableText"]}
  }]], "3ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, None, None, None, None, {None}}, 
   "RowsIndexed" -> {}},
 GridBoxDividers->{
  "Rows" -> {{True, True, True, True, True, True, True, True}}}},
 CellID->50244769],

Cell["\<\
Symmetry of the orientation axes is used whenever it is possible to limit the \
use of memory and decrease the computation time from processing steps.\
\>", "Notes",
 CellID->525793994],

Cell["", "SectionFooterSpacer"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{6},
   "Spacer1"]]],
 "Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(21)", "ExampleCount"]
}], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->1],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(2)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1],

Cell["\<\
By using all default settings a cake wavelet stack can be created by only \
providing the number of orientations\
\>", "ExampleText",
 CellID->16962313],

Cell[BoxData[{
 RowBox[{"Needs", "[", "\"\<LieAnalysis`\>\"", "]"}], "\[IndentingNewLine]", 
 RowBox[{"CakeWaveletStack", "[", 
  RowBox[{"12", ",", " ", "21"}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->5496259],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->513374387],

Cell["The shape of the cake wavelets can be modified using options", \
"ExampleText",
 CellID->1841512310],

Cell[BoxData[
 RowBox[{"CakeWaveletStack", "[", 
  RowBox[{"12", ",", "21", ",", 
   RowBox[{"\"\<InflectionPoint\>\"", "\[Rule]", ".4"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->599109829],

Cell["To see the individual wavelets", "ExampleText",
 CellID->1450779784],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ImageAdjust", "[", 
    RowBox[{"Image", "[", "#", "]"}], "]"}], "&"}], "/@", 
  RowBox[{"Re", "@", 
   RowBox[{
    RowBox[{"CakeWaveletStack", "[", 
     RowBox[{"12", ",", "51"}], "]"}], "[", "\"\<FullData\>\"", 
    "]"}]}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1552683145]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Scope",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(3)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1293636265],

Cell["\<\
CakeWaveletStack returns a ObjCakeWavelet object, which is the child of \
ObjOrientationWavelet\
\>", "ExampleText",
 CellID->197339768],

Cell[BoxData[
 RowBox[{"ValidQ", "[", 
  RowBox[{
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"2", ",", "22"}], "]"}], ",", 
   "LieAnalysis`Objects`ObjOrientationWavelet`ObjOrientationWavelet"}], 
  "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->39365668],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->20679480],

Cell["\<\
Directional cake wavelets are Cake Wavelets of which the tail is cut of (only \
the head is left). It is also good to note that by adding the wavelet at \
\[Theta] = 0 and \[Theta] = \[Pi], the original wavelet is reconstructed.\
\>", "ExampleText",
 CellID->104945823],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cakeWavelets", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{
     RowBox[{"n", "=", "24"}], ",", "21", ",", 
     RowBox[{"\"\<Directional\>\"", "\[Rule]", "True"}], ",", 
     RowBox[{"\"\<DcStandardDeviation\>\"", "\[Rule]", "100"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GraphicsRow", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"ImageAdjust", "@", 
      RowBox[{"Image", "[", 
       RowBox[{"Re", "[", "#", "]"}], "]"}]}], "&"}], "/@", 
    RowBox[{
    "cakeWavelets", "\[LeftDoubleBracket]", "\"\<Data\>\"", 
     "\[RightDoubleBracket]"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "Full"}]}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->191649833]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->53330947],

Cell["\<\
The shape of the wavelets in the Fourier domain can be controlled by the \
options: InflictionPoint, SplineOrder, MnOrder, OverlapFactor and \
DcStandardDeviation.\
\>", "ExampleText",
 CellID->229599025],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CakeWavelets", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"CakeWaveletStack", "[", 
      RowBox[{
       RowBox[{"n", "=", "12"}], ",", "75", ",", 
       RowBox[{"\"\<SplineOrder\>\"", "\[Rule]", 
        RowBox[{"#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
        ",", " ", 
       RowBox[{"\"\<MnOrder\>\"", "\[Rule]", 
        RowBox[{"#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
        ",", " ", 
       RowBox[{"\"\<OverlapFactor\>\"", "\[Rule]", 
        RowBox[{"#", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}]}],
        ",", 
       RowBox[{"\"\<DcStandardDeviation\>\"", "\[Rule]", 
        RowBox[{"#", "\[LeftDoubleBracket]", "4", "\[RightDoubleBracket]"}]}],
        ",", 
       RowBox[{"\"\<InflectionPoint\>\"", "\[Rule]", 
        RowBox[{
        "#", "\[LeftDoubleBracket]", "5", "\[RightDoubleBracket]"}]}]}], 
      "]"}], "&"}], "/@", 
    RowBox[{"{", "\n", "\t", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"3", ",", "8", ",", "1", ",", "3", ",", "0.7"}], "}"}], ",", 
      "\n", "\t", 
      RowBox[{"{", 
       RowBox[{"10", ",", "8", ",", "1", ",", "3", ",", "0.7"}], "}"}], ",", 
      "\n", "\t", 
      RowBox[{"{", 
       RowBox[{"3", ",", "3", ",", "1", ",", "3", ",", "0.7"}], "}"}], ",", 
      "\n", "\t", 
      RowBox[{"{", 
       RowBox[{"3", ",", "8", ",", "4", ",", "3", ",", "0.7"}], "}"}], ",", 
      "\n", "\t", 
      RowBox[{"{", 
       RowBox[{"3", ",", "8", ",", "1", ",", "20", ",", "0.7"}], "}"}], ",", 
      "\n", "\t", 
      RowBox[{"{", 
       RowBox[{"3", ",", "8", ",", "1", ",", "8", ",", "0.2"}], "}"}]}], "\n",
      "}"}]}]}], ";"}], "\n", 
 RowBox[{"GraphicsGrid", "[", 
  RowBox[{
   RowBox[{"{", "\n", "\t", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\"\<\>\"", ",", "\"\<Default\>\"", ",", 
       "\"\<SplineOrder = 10 \\n (default = 3)\>\"", ",", 
       "\"\<MnOrder = 3 \\n (default = 8)\>\"", ",", 
       "\"\<OverlapFactor = 4 \\n (default = 1)\>\"", ",", 
       "\"\<DcStd = 20 \\n (default = 3)\>\"", ",", 
       "\"\<InflectionPoint = 0.2 \\n (default = 0.7)\>\""}], "}"}], ",", 
     "\n", "\t", 
     RowBox[{"Prepend", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Image", "[", 
          RowBox[{"Total", "[", 
           RowBox[{
           "#", "\[LeftDoubleBracket]", "\"\<FourierCake\>\"", 
            "\[RightDoubleBracket]"}], "]"}], "]"}], "&"}], "/@", 
        "CakeWavelets"}], ",", "\"\<FourierCake\>\""}], "]"}], ",", "\n", 
     "\t", 
     RowBox[{"Prepend", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Image", "[", 
          RowBox[{
           RowBox[{
           "#", "\[LeftDoubleBracket]", "\"\<FourierCake\>\"", 
            "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", "1", 
           "\[RightDoubleBracket]"}], "]"}], "&"}], "/@", "CakeWavelets"}], 
       ",", " ", "\"\<FourierSlice\>\""}], "]"}], ",", "\n", "\t", 
     RowBox[{"Prepend", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"ImageAdjust", "@", 
          RowBox[{"Image", "[", 
           RowBox[{"Re", "[", 
            RowBox[{
             RowBox[{
             "#", "\[LeftDoubleBracket]", "\"\<Data\>\"", 
              "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", "1", 
             "\[RightDoubleBracket]"}], "]"}], "]"}]}], "&"}], "/@", 
        "CakeWavelets"}], ",", "\"\<Spatial Wavelet \[Theta] = 0\>\""}], 
      "]"}]}], "\n", "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "Full"}]}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->432553630]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Generalizations & Extensions",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(2)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1020263627],

Cell[TextData[{
 "You can manipulate the ",
 StyleBox["elements",
  FontSlant->"Italic"],
 " of the ObjCakeWavelet using Affix. Be careful when you do this as ",
 StyleBox["elements",
  FontSlant->"Italic"],
 " might become inconsistent, this can lead to unexpected results."
}], "ExampleText",
 CellID->560092696],

Cell[BoxData[
 RowBox[{"Affix", "[", 
  RowBox[{
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "11"}], "]"}], ",", 
   RowBox[{"\"\<Symmetry\>\"", "\[Rule]", 
    RowBox[{"2", "\[Pi]"}]}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->785189421],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->998836185],

Cell[TextData[{
 "To create your own orientation wavelet which can be used by the LieAnalysis \
package, you can use the ObjOrientationWavelet format. Do set all ",
 StyleBox["elements ",
  FontSlant->"Italic"],
 "consistently."
}], "ExampleText",
 CellID->1046902088],

Cell[BoxData[
 RowBox[{"ObjOrientationWavelet", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<Data\>\"", "\[Rule]", 
    RowBox[{"RandomReal", "[", 
     RowBox[{"1", ",", 
      RowBox[{"{", 
       RowBox[{"6", ",", "11", ",", "11"}], "}"}]}], "]"}]}], "}"}], 
  "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1158253530]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Options",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(7)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->2061341341],

Cell[CellGroupData[{

Cell[TextData[{
 "Directional",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->345248695],

Cell["\<\
The tails and head of the wavelet can be separated into two layers by setting \
this option to True. Note that the \
\>", "ExampleText",
 CellID->258392697],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cws", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "23", ",", 
     RowBox[{"\"\<Directional\>\"", "\[Rule]", "True"}], ",", 
     RowBox[{"\"\<DcStandardDeviation\>\"", "\[Rule]", "4"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GraphicsRow", "@", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"ImageAdjust", "@", 
      RowBox[{"Image", "[", 
       RowBox[{"Re", "[", "#", "]"}], "]"}]}], "&"}], "/@", 
    RowBox[{
    "cws", "\[LeftDoubleBracket]", "\"\<Data\>\"", 
     "\[RightDoubleBracket]"}]}], ")"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->200828983]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Design",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->64022664],

Cell[TextData[{
 "The default design used is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "\[Psi]"], "=", 
    RowBox[{
     RowBox[{"\[Integral]", 
      RowBox[{"\[VerticalSeparator]", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["\[Psi]", "^"], "n"], "(", "\[Omega]", ")"}], 
       "\[VerticalSeparator]", " ", 
       RowBox[{"d\[Theta]", "(", "n", ")"}]}]}], " ", "=", " ", "1"}]}], 
   TraditionalForm]]],
 ". Another design that can be used is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["M", "\[Psi]"], "=", 
    RowBox[{
     RowBox[{"\[Integral]", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[VerticalSeparator]", 
          RowBox[{
           SubscriptBox[
            OverscriptBox["\[Psi]", "^"], "n"], "(", "\[Omega]", ")"}], " ", 
          "\[VerticalSeparator]"}], ")"}], "2"], 
       RowBox[{"d\[Theta]", "(", "n", ")"}]}]}], " ", "=", " ", "1"}]}], 
   TraditionalForm]]],
 ". Note that the reconstruction method depends on this setting (See \
OrientationScoreTransform)."
}], "ExampleText",
 CellID->6464364],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cws1", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "23", ",", 
     RowBox[{"\"\<Design\>\"", "\[Rule]", "\"\<N\>\""}]}], "]"}]}], ";", " ", 
  
  RowBox[{"cws2", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "23", ",", 
     RowBox[{"\"\<Design\>\"", "\[Rule]", "\"\<M\>\""}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GraphicsRow", "@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws1", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "1", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}], ",", 
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws2", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "1", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}]}], 
   "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->55042216]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "SplineOrder",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->120190943],

Cell["\<\
The spline order indicates the order of the used B-spline to construct the \
wavelets. A higher order means more overlap between the wavelets, which means \
that each orientation layer is responsive to a wider range of orientations. \
\>", "ExampleText",
 CellID->244132258],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cws1", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "23", ",", 
     RowBox[{"\"\<SplineOrder\>\"", "\[Rule]", "8"}]}], "]"}]}], ";", " ", 
  RowBox[{"cws2", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "23", ",", 
     RowBox[{"\"\<SplineOrder\>\"", "\[Rule]", "3"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GraphicsRow", "@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws1", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "1", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}], ",", 
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws2", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "1", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}]}], 
   "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->81614867]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "InflectionPoint",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->270547551],

Cell[TextData[{
 "The inflection point indicates the cut off for high frequencies (Gaussian \
decay). Note that this should be sufficiently small that the cake has a \
smooth ending towards the edge of the image. The gaussian decay window is \
described as follows:\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalM]", "N"], "(", 
     RowBox[{
      SuperscriptBox["\[Rho]", "2"], 
      SuperscriptBox["t", 
       RowBox[{"-", "1"}]]}], ")"}], "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["\[Rho]", "2"], "t"]}]], " ", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"k", "=", "0"}], "N"], 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[Rho]", "2"], 
           SuperscriptBox["t", 
            RowBox[{"-", "1"}]]}], ")"}], "k"], 
        RowBox[{"k", "!"}]]}]}], "\[LessEqual]", " ", "1"}]}], 
   TraditionalForm]]],
 ", in which \[Rho] denotes the inflection point and N is the order of the \
Taylor series (see option ",
 StyleBox["MnOrder", "TI"],
 ")."
}], "ExampleText",
 CellID->58903934],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cws1", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "45"}], "]"}]}], ";", " ", 
  RowBox[{"cws2", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "45", ",", 
     RowBox[{"\"\<InflectionPoint\>\"", "\[Rule]", "0.3"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GraphicsRow", "@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws1", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "1", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}], ",", 
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws2", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "1", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}]}], 
   "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->63329269]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "MnOrder",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->327580085],

Cell[TextData[{
 "The order of the Taylor series, to ensure slower (gaussian) decay. The \
higher the order the quicker the decay is. The Gaussian window is described \
by: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[ScriptCapitalM]", "N"], "(", 
     RowBox[{
      SuperscriptBox["\[Rho]", "2"], 
      SuperscriptBox["t", 
       RowBox[{"-", "1"}]]}], ")"}], "=", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"-", 
        FractionBox[
         SuperscriptBox["\[Rho]", "2"], "t"]}]], " ", 
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"k", "=", "0"}], "N"], 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["\[Rho]", "2"], 
           SuperscriptBox["t", 
            RowBox[{"-", "1"}]]}], ")"}], "k"], 
        RowBox[{"k", "!"}]]}]}], "\[LessEqual]", " ", "1"}]}], 
   TraditionalForm]]],
 ", in which N denotes the order and \[Rho] the inflection point (See option \
inflectionPoint). This best can be seen in the Fourier domain."
}], "ExampleText",
 CellID->462540913],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cws1", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "23"}], "]"}]}], ";", " ", 
  RowBox[{"cws2", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "23", ",", 
     RowBox[{"\"\<MnOrder\>\"", "\[Rule]", "50"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"GraphicsRow", "@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Image", "[", 
     RowBox[{"Total", "[", 
      RowBox[{
      "cws1", "\[LeftDoubleBracket]", "\"\<FourierCake\>\"", 
       "\[RightDoubleBracket]"}], "]"}], "]"}], ",", 
    RowBox[{"Image", "[", 
     RowBox[{"Total", "[", 
      RowBox[{
      "cws2", "\[LeftDoubleBracket]", "\"\<FourierCake\>\"", 
       "\[RightDoubleBracket]"}], "]"}], "]"}]}], "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->340232891]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "DcStandardDeviation",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->817945707],

Cell["\<\
The DC filter is used to remove the low-frequency component from the \
Cake-Wavelets. This is required to remove long tails in the spatial domain. \
It also ensures that the cake pieces are smooth in the Fourier domain. This \
can be best viewed in the Fourier domain. The option value is the standard \
deviation (in the spatial domain) of a gaussian that is to remove the low \
frequencies. \
\>", "ExampleText",
 CellID->80729054],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cws1", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "45", ",", 
     RowBox[{"\"\<DcStandardDeviation\>\"", "\[Rule]", "12"}]}], "]"}]}], ";",
   " ", 
  RowBox[{"cws2", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "45"}], "]"}]}], ";"}], "\n", 
 RowBox[{"GraphicsRow", "@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Image", "[", 
     RowBox[{"Total", "[", 
      RowBox[{
      "cws1", "\[LeftDoubleBracket]", "\"\<FourierCake\>\"", 
       "\[RightDoubleBracket]"}], "]"}], "]"}], ",", 
    RowBox[{"Image", "[", 
     RowBox[{"Total", "[", 
      RowBox[{
      "cws2", "\[LeftDoubleBracket]", "\"\<FourierCake\>\"", 
       "\[RightDoubleBracket]"}], "]"}], "]"}]}], "}"}]}], "\n", 
 RowBox[{"GraphicsRow", "@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws1", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "1", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}], ",", 
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws2", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "1", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}]}], 
   "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->281889816]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "OverlapFactor",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->4964550],

Cell["\<\
The overlap factor is used to sub-sample the fourier spectrum in angular \
direction without narrowing the cake pieces. The resulting Orientation Score \
(OS) has smoother transitions between layers in \[Theta]-direction. \
\>", "ExampleText",
 CellID->197374881],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cws1", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "45"}], "]"}]}], ";", " ", 
  RowBox[{"cws2", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "45", ",", 
     RowBox[{"\"\<OverlapFactor\>\"", "\[Rule]", "2"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{"GraphicsRow", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"ImageAdjust", "@", 
       RowBox[{"Image", "[", 
        RowBox[{"Re", "[", "#", "]"}], "]"}]}], "&"}], "/@", 
     RowBox[{"cws1", "[", 
      RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}]}], ")"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", "Full"}]}], "]"}], "\n", 
 RowBox[{"GraphicsRow", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"ImageAdjust", "@", 
       RowBox[{"Image", "[", 
        RowBox[{"Re", "[", "#", "]"}], "]"}]}], "&"}], "/@", 
     RowBox[{"cws2", "[", 
      RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}]}], ")"}], ",", " ", 
   RowBox[{"ImageSize", "\[Rule]", "Full"}]}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->68126647]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Applications",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->258228157],

Cell[TextData[{
 "The cake wavelets can be used to compute an orientation score using the \
function ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["OrientationScoreTransform"]],
    "paclet:LieAnalysis/ref/OrientationScoreTransform"},
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"]
}], "ExampleText",
 CellID->292693289],

Cell[BoxData[
 RowBox[{"OrientationScoreTransform", "[", 
  RowBox[{"LieAnalysisData", "[", 
   RowBox[{"{", 
    RowBox[{"\"\<image\>\"", ",", "\"\<euler-spirals\>\""}], "}"}], "]"}], 
  "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->499503895]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Properties & Relations",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(5)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->2123667759],

Cell[TextData[{
 "To get the data you can request the element ",
 StyleBox["data ",
  FontSlant->"Italic"],
 "which is stored within the object."
}], "ExampleText",
 CellID->385991406],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{
     RowBox[{"n", " ", "=", " ", "12"}], ",", " ", "21"}], "]"}], 
   "\[LeftDoubleBracket]", "\"\<Data\>\"", "\[RightDoubleBracket]"}], "//", 
  "Dimensions"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->91228733],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->7687332],

Cell[TextData[{
 "To disband the object and get all stored information as an ",
 ButtonBox["Association",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Association"],
 " the ",
 ButtonBox["Normal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Normal"],
 " function can be applied. "
}], "ExampleText",
 CellID->1010077310],

Cell[BoxData[
 RowBox[{"Framed", "/@", 
  RowBox[{"Keys", "[", 
   RowBox[{"Normal", "[", "cakeWavelets", "]"}], "]"}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->7659918]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->1238722435],

Cell["\<\
Elements that can be stored in the object can be found through the property \
defaults. \
\>", "ExampleText",
 CellID->1392205060],

Cell[BoxData[
 RowBox[{"Framed", "/@", 
  RowBox[{
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "21"}], "]"}], "[", "\"\<Elements\>\"", 
   "]"}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2030601230]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->1501491027],

Cell["\<\
The output object contains many properties of the cake wavelets. These \
properties are computed when they are requested. The available properties can \
be found by the property properties.\
\>", "ExampleText",
 CellID->479354892],

Cell[BoxData[
 RowBox[{"Framed", "/@", 
  RowBox[{
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "11"}], "]"}], "[", "\"\<Properties\>\"", 
   "]"}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->894990216]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->215688728],

Cell["\<\
Symmetry in the orientation axes is used whenever this is possible. For \
example when we request 12 orientations only 6 orientations are stored and \
the element symmetry is set to \[Pi].\
\>", "ExampleText",
 CellID->128055360],

Cell[BoxData[
 RowBox[{"cws", " ", "=", " ", 
  RowBox[{"CakeWaveletStack", "[", 
   RowBox[{"12", ",", "11"}], "]"}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->262354246],

Cell[BoxData[
 RowBox[{
  RowBox[{"cws", "[", 
   RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "//", "Dimensions"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1520002547],

Cell[TextData[{
 "To get the data without using the property, you can use the property Full",
 StyleBox["Data ",
  FontSlant->"Italic"],
 "instead of the element."
}], "ExampleText",
 CellID->688646404],

Cell[BoxData[
 RowBox[{
  RowBox[{"cws", "[", "\"\<FullData\>\"", "]"}], "//", 
  "Dimensions"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->787859925],

Cell["\<\
If you use an odd number of orientations or directional wavelets it is no \
longer possible to save memory by using symmetry.\
\>", "ExampleText",
 CellID->319551712],

Cell[BoxData[
 RowBox[{
  RowBox[{"CakeWaveletStack", "[", 
   RowBox[{"13", ",", "11"}], "]"}], "\[LeftDoubleBracket]", 
  "\"\<Symmetry\>\"", "\[RightDoubleBracket]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->993691099],

Cell[BoxData[
 RowBox[{
  RowBox[{"CakeWaveletStack", "[", 
   RowBox[{"12", ",", "11", ",", 
    RowBox[{"\"\<Directional\>\"", "\[Rule]", "True"}]}], "]"}], 
  "\[LeftDoubleBracket]", "\"\<Symmetry\>\"", 
  "\[RightDoubleBracket]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->1952979937]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Possible Issues",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1305812373],

Cell["\<\
Filters look unstable. This could be due that the Fourier spectrum is not \
smooth.\
\>", "ExampleText",
 CellID->98070909],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cws", " ", "=", " ", 
   RowBox[{"CakeWaveletStack", "[", 
    RowBox[{"12", ",", "31", ",", 
     RowBox[{"\"\<InflectionPoint\>\"", "\[Rule]", "1"}], ",", 
     RowBox[{"\"\<MnOrder\>\"", "\[Rule]", "3"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{"GraphicsRow", "@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ImageAdjust", "[", 
     RowBox[{"Image", "[", 
      RowBox[{"Re", "[", 
       RowBox[{
        RowBox[{"cws", "[", 
         RowBox[{"[", "\"\<Data\>\"", "]"}], "]"}], "\[LeftDoubleBracket]", 
        "3", "\[RightDoubleBracket]"}], "]"}], "]"}], "]"}], ",", 
    RowBox[{"Image", "[", 
     RowBox[{"Total", "[", 
      RowBox[{
      "cws", "\[LeftDoubleBracket]", "\"\<FourierCake\>\"", 
       "\[RightDoubleBracket]"}], "]"}], "]"}]}], "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->377479142]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["", "SectionFooterSpacer"],

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{6},
   "Spacer1"]]],
 "See Also"
}], "SeeAlsoSection",
 WholeCellGroupOpener->True,
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["GaborWaveletStack"]],
    "paclet:LieAnalysis/ref/GaborWaveletStack"},
   "RefLink",
   BaseStyle->{"InlineFormula", FontFamily -> "Verdana"}]], "InlineFormula"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["CakeWaveletStack3D"]],
    "paclet:LieAnalysis/ref/CakeWaveletStack3D"},
   "RefLink",
   BaseStyle->{"InlineFormula", FontFamily -> "Verdana"}]], "InlineFormula"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["OrientationScoreTransform"]],
    "paclet:LieAnalysis/ref/OrientationScoreTransform"},
   "RefLink",
   BaseStyle->{"InlineFormula", FontFamily -> "Verdana"}]], "InlineFormula"]
}], "SeeAlso",
 CellID->929782353]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{6},
   "Spacer1"]]],
 "Tutorials"
}], "TutorialsSection",
 WholeCellGroupOpener->True,
 CellID->250839057],

Cell["", "SectionHeaderSpacer"],

Cell[BoxData[
 TemplateBox[{Cell[
    TextData["Basic Usage"]],"paclet:LieAnalysis/tutorial/BasicUsage"},
  "RefLink",
  BaseStyle->"Tutorials"]], "Tutorials",
 CellID->1196281537],

Cell["", "SectionFooterSpacer"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{6},
   "Spacer1"]]],
 "Related Guides"
}], "MoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->38303248],

Cell["", "SectionHeaderSpacer"],

Cell[BoxData[
 TemplateBox[{Cell[
    TextData["LieAnalysis Package"]],"paclet:LieAnalysis/guide/LieAnalysis"},
  "RefLink",
  BaseStyle->"MoreAbout"]], "MoreAbout",
 CellID->1]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"CakeWaveletStack",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "LieAnalysis Package", ButtonData -> 
         "paclet:LieAnalysis/guide/LieAnalysis", 
         BaseStyle -> {"Link", "DockedLinkTrail"}]}]}}, ColumnAlignments -> 
    Left], "HasOptions" -> False, "ExampleCounter" -> 1, "NeedPlatMsgIn" -> 
  None, "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2017, 10, 26, 15, 6, 47.5252416}", 
    "history" -> {"0.2", "0.4.1", "", ""}, "context" -> "LieAnalysis`", 
    "keywords" -> {
     "Orientation Scores", "Cake Wavelets", "Orientation Wavelets", 
      "LieAnalysis", "OrientationScoreTransform"}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Lie Analysis Symbol", "language" -> "en", "paclet" -> "LieAnalysis", 
    "status" -> "None", "summary" -> 
    "CakeWaveletStack[n, size] constructs a Cake Wavelet stack having n \
orientations, dimensions size x size with default parameters. \
CakeWaveletStack[n, size, options] constructs a Cake Wavelet stack having n \
orientations, dimensions size x size with the specified options.", 
    "synonyms" -> {}, "tabletags" -> {}, "title" -> "CakeWaveletStack", 
    "titlemodifier" -> "", "windowtitle" -> "CakeWaveletStack", "type" -> 
    "Symbol", "uri" -> "LieAnalysis/ref/CakeWaveletStack"}},
CellContext->"Global`",
FrontEndVersion->"11.2 for Microsoft Windows (64-bit) (September 10, 2017)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> 
  "11.2 for Microsoft Windows (64-bit) (September 10, 2017)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[14621, 447, 260, 10, 70, "PrimaryExamplesSection",ExpressionUUID->"c9ac5d9c-f2ca-44b8-845f-be4e71f2c858",
   CellTags->"PrimaryExamplesSection",
   CellID->1]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 48193, 1595}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[588, 21, 4543, 113, 70, "AnchorBarGrid",ExpressionUUID->"613b50e2-703f-44ef-ac27-eea0546d7052",
 CellID->1],
Cell[5134, 136, 173, 6, 70, "ObjectNameGrid",ExpressionUUID->"ab1a5820-609e-4b2c-ab52-83e65f007843"],
Cell[5310, 144, 1128, 36, 70, "Usage",ExpressionUUID->"27e9697c-5908-431e-a749-90e56ccacfe7",
 CellID->982511436],
Cell[CellGroupData[{
Cell[6463, 184, 621, 19, 70, "NotesSection",ExpressionUUID->"3bead49f-3552-44c6-8e12-eb99102f688d",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->1],
Cell[7087, 205, 31, 0, 70, "SectionHeaderSpacer",ExpressionUUID->"ad08bcfb-2af8-4e8d-abba-76dbe8048a1b"],
Cell[7121, 207, 581, 17, 70, "Notes",ExpressionUUID->"8cf6cd09-4fa8-4673-bc6e-fe6c02eb282d",
 CellID->1067943069],
Cell[7705, 226, 4212, 137, 70, "Notes",ExpressionUUID->"13067b75-25b8-46e2-9f0b-7380de37e82c",
 CellID->233796496],
Cell[11920, 365, 454, 17, 70, "Notes",ExpressionUUID->"60744cf3-bf7d-481a-b659-07e2c93e4311",
 CellID->359789379],
Cell[12377, 384, 369, 13, 70, "Notes",ExpressionUUID->"f57bdee0-b43e-41ec-9922-0698f39ea599",
 CellID->353028093],
Cell[12749, 399, 72, 1, 70, "Notes",ExpressionUUID->"0c5a4816-12af-4f2e-923e-c30197fa0681",
 CellID->507720613],
Cell[12824, 402, 1528, 32, 70, "3ColumnTableMod",ExpressionUUID->"0a403e00-b164-40de-a869-5dd17550d6bd",
 CellID->50244769],
Cell[14355, 436, 195, 4, 70, "Notes",ExpressionUUID->"e1ea3ef5-be07-4437-b53e-967e10b86431",
 CellID->525793994],
Cell[14553, 442, 31, 0, 70, "SectionFooterSpacer",ExpressionUUID->"b6a335d9-2441-45aa-9b07-9f4b1028e9c5"]
}, Closed]],
Cell[CellGroupData[{
Cell[14621, 447, 260, 10, 70, "PrimaryExamplesSection",ExpressionUUID->"c9ac5d9c-f2ca-44b8-845f-be4e71f2c858",
 CellTags->"PrimaryExamplesSection",
 CellID->1],
Cell[CellGroupData[{
Cell[14906, 461, 187, 6, 70, "ExampleSection",ExpressionUUID->"d415a6e9-3e43-4a88-8af1-b8eec6b7aacc",
 CellID->1],
Cell[15096, 469, 162, 4, 70, "ExampleText",ExpressionUUID->"f7b7ada8-be5d-4254-9ad8-ae2847c2bfe6",
 CellID->16962313],
Cell[15261, 475, 220, 5, 70, "Input",ExpressionUUID->"58c3bf82-db2d-488e-ac33-0d76fb2177f6",
 CellID->5496259],
Cell[CellGroupData[{
Cell[15506, 484, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"313ad786-1d97-4c73-b689-fe4d11475c19",
 CellID->513374387],
Cell[15634, 489, 106, 2, 70, "ExampleText",ExpressionUUID->"ce832b02-365c-4d54-b353-5e2ddff08a79",
 CellID->1841512310],
Cell[15743, 493, 200, 5, 70, "Input",ExpressionUUID->"17d1e7a1-92d5-4699-b06a-66ebceba9db7",
 CellID->599109829],
Cell[15946, 500, 74, 1, 70, "ExampleText",ExpressionUUID->"d4a0ab1e-f1ca-4088-bc6d-8e4a7ab1b753",
 CellID->1450779784],
Cell[16023, 503, 331, 11, 70, "Input",ExpressionUUID->"79ee8fb7-103d-46e3-9928-6bb302312aa0",
 CellID->1552683145]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16403, 520, 187, 6, 70, "ExampleSection",ExpressionUUID->"67892423-7842-41e4-807c-cd48249102b0",
 CellID->1293636265],
Cell[16593, 528, 146, 4, 70, "ExampleText",ExpressionUUID->"02571c13-8119-4adc-9d27-aad6450192be",
 CellID->197339768],
Cell[16742, 534, 263, 8, 70, "Input",ExpressionUUID->"b8171ba8-8669-48d6-b41b-15a123d2c1df",
 CellID->39365668],
Cell[CellGroupData[{
Cell[17030, 546, 124, 3, 70, "ExampleDelimiter",ExpressionUUID->"49fff66a-5f0e-4bfe-a83b-57ad81f34a49",
 CellID->20679480],
Cell[17157, 551, 279, 5, 70, "ExampleText",ExpressionUUID->"b1fddc70-3aa1-44e4-9313-cd4063f47629",
 CellID->104945823],
Cell[17439, 558, 731, 21, 70, "Input",ExpressionUUID->"437a4d10-7a55-4de4-9bfb-56ec338cc05e",
 CellID->191649833]
}, Open  ]],
Cell[CellGroupData[{
Cell[18207, 584, 124, 3, 70, "ExampleDelimiter",ExpressionUUID->"9d3bfbd6-bdcd-4666-8350-4939b8596e97",
 CellID->53330947],
Cell[18334, 589, 214, 5, 70, "ExampleText",ExpressionUUID->"b14f50b0-80d3-4bff-a80d-27cbc60ae78d",
 CellID->229599025],
Cell[18551, 596, 3633, 95, 70, "Input",ExpressionUUID->"82ce9b91-11d4-4050-949c-36d02ed5aee3",
 CellID->432553630]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[22233, 697, 210, 6, 70, "ExampleSection",ExpressionUUID->"4435427c-f991-4dc5-b6a7-310549268e3c",
 CellID->1020263627],
Cell[22446, 705, 314, 9, 70, "ExampleText",ExpressionUUID->"540d7c9a-a1c8-4c2d-a10e-834ea2b66f11",
 CellID->560092696],
Cell[22763, 716, 264, 8, 70, "Input",ExpressionUUID->"dcd8d905-f68c-4374-8b52-0c4c08ac0061",
 CellID->785189421],
Cell[CellGroupData[{
Cell[23052, 728, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"9b9258f7-a826-4627-901a-308654f8e921",
 CellID->998836185],
Cell[23180, 733, 268, 7, 70, "ExampleText",ExpressionUUID->"e186932f-26bd-4f4f-8d8c-62db02c1385c",
 CellID->1046902088],
Cell[23451, 742, 317, 10, 70, "Input",ExpressionUUID->"1b94d196-1b3d-45e7-af5d-bfc3fb187530",
 CellID->1158253530]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[23817, 758, 189, 6, 70, "ExampleSection",ExpressionUUID->"21b012f7-74ee-425b-873f-0e44f45c011d",
 CellID->2061341341],
Cell[CellGroupData[{
Cell[24031, 768, 198, 6, 70, "ExampleSubsection",ExpressionUUID->"60d53104-9b9d-4ab3-ab35-a897454183de",
 CellID->345248695],
Cell[24232, 776, 166, 4, 70, "ExampleText",ExpressionUUID->"a3943300-fc04-4831-9d1b-970245e6238e",
 CellID->258392697],
Cell[24401, 782, 639, 19, 70, "Input",ExpressionUUID->"8f80d204-a65e-4378-9c3c-e3494a5dfccd",
 CellID->200828983]
}, Closed]],
Cell[CellGroupData[{
Cell[25077, 806, 192, 6, 70, "ExampleSubsection",ExpressionUUID->"0c57eed8-42e9-48bf-92e0-61da769cc618",
 CellID->64022664],
Cell[25272, 814, 1116, 35, 70, "ExampleText",ExpressionUUID->"5c797049-f0c6-4c3e-9b2b-1970dc1c692a",
 CellID->6464364],
Cell[26391, 851, 1077, 31, 70, "Input",ExpressionUUID->"14681c60-a581-4a65-8722-b5588aa783ff",
 CellID->55042216]
}, Closed]],
Cell[CellGroupData[{
Cell[27505, 887, 198, 6, 70, "ExampleSubsection",ExpressionUUID->"61a717f6-7f55-41dd-b3cd-a8928395afe1",
 CellID->120190943],
Cell[27706, 895, 284, 5, 70, "ExampleText",ExpressionUUID->"e3bd7d6d-894c-4645-af20-d62a8e532c80",
 CellID->244132258],
Cell[27993, 902, 1068, 30, 70, "Input",ExpressionUUID->"2195cf89-136d-4b55-ae8c-bec8ff8f5854",
 CellID->81614867]
}, Closed]],
Cell[CellGroupData[{
Cell[29098, 937, 202, 6, 70, "ExampleSubsection",ExpressionUUID->"9c369edd-9410-4da7-b57a-f15f4cf1c10c",
 CellID->270547551],
Cell[29303, 945, 1230, 37, 70, "ExampleText",ExpressionUUID->"2e3dffef-d576-4ae4-ac95-b1b9d322a2c3",
 CellID->58903934],
Cell[30536, 984, 1014, 29, 70, "Input",ExpressionUUID->"02c6ea1c-cd48-4235-a072-f93f5069f75e",
 CellID->63329269]
}, Closed]],
Cell[CellGroupData[{
Cell[31587, 1018, 194, 6, 70, "ExampleSubsection",ExpressionUUID->"f59dc122-ce19-48de-b4bf-af15312c7c1c",
 CellID->327580085],
Cell[31784, 1026, 1140, 34, 70, "ExampleText",ExpressionUUID->"4a3eb165-cadb-4f53-b30d-250a84491e02",
 CellID->462540913],
Cell[32927, 1062, 817, 23, 70, "Input",ExpressionUUID->"6014ae88-2309-4432-99f3-7851c510be3f",
 CellID->340232891]
}, Closed]],
Cell[CellGroupData[{
Cell[33781, 1090, 206, 6, 70, "ExampleSubsection",ExpressionUUID->"c9f23a30-4ce6-40b4-b6eb-4c9735a3190e",
 CellID->817945707],
Cell[33990, 1098, 443, 8, 70, "ExampleText",ExpressionUUID->"00df4163-b59c-4dae-92ce-bc4cf6568ae3",
 CellID->80729054],
Cell[34436, 1108, 1460, 42, 70, "Input",ExpressionUUID->"3e6c3972-0b3e-4a9c-bbc5-d5a96ea6b3d9",
 CellID->281889816]
}, Closed]],
Cell[CellGroupData[{
Cell[35933, 1155, 198, 6, 70, "ExampleSubsection",ExpressionUUID->"cd0f4b8a-b8b5-4bad-a114-9c1c820313e7",
 CellID->4964550],
Cell[36134, 1163, 273, 5, 70, "ExampleText",ExpressionUUID->"4b1fc55a-fa15-4a6e-98b2-83eddf2ac123",
 CellID->197374881],
Cell[36410, 1170, 1115, 33, 70, "Input",ExpressionUUID->"b2146f4f-aeb0-4bd8-be56-4024ffd6aeef",
 CellID->68126647]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[37574, 1209, 193, 6, 70, "ExampleSection",ExpressionUUID->"260c77b7-2f7e-47d0-9fc4-2005b543d042",
 CellID->258228157],
Cell[37770, 1217, 346, 10, 70, "ExampleText",ExpressionUUID->"93c347a2-ee3c-44cc-8f8d-0d0179fc27d7",
 CellID->292693289],
Cell[38119, 1229, 247, 7, 70, "Input",ExpressionUUID->"3107655e-1766-463a-beee-fac1f1dac1c7",
 CellID->499503895]
}, Closed]],
Cell[CellGroupData[{
Cell[38403, 1241, 204, 6, 70, "ExampleSection",ExpressionUUID->"9acc0651-78fb-46d0-98b6-fd4bcb6def75",
 CellID->2123667759],
Cell[38610, 1249, 184, 6, 70, "ExampleText",ExpressionUUID->"cc69151f-155f-44c0-b073-72adafdec0cf",
 CellID->385991406],
Cell[38797, 1257, 298, 9, 70, "Input",ExpressionUUID->"87de8899-d7c3-43aa-b7d9-b82e06d02309",
 CellID->91228733],
Cell[CellGroupData[{
Cell[39120, 1270, 123, 3, 70, "ExampleDelimiter",ExpressionUUID->"857cea64-476c-4d65-b95e-97444f1eec83",
 CellID->7687332],
Cell[39246, 1275, 325, 11, 70, "ExampleText",ExpressionUUID->"d395da8f-3330-41ce-91df-7595ae42476e",
 CellID->1010077310],
Cell[39574, 1288, 172, 5, 70, "Input",ExpressionUUID->"4fe3a6d4-c92d-4906-9d4d-5ede65d7caf1",
 CellID->7659918]
}, Open  ]],
Cell[CellGroupData[{
Cell[39783, 1298, 126, 3, 70, "ExampleDelimiter",ExpressionUUID->"4acf6d29-2479-4acd-a563-b73a8f1535d7",
 CellID->1238722435],
Cell[39912, 1303, 140, 4, 70, "ExampleText",ExpressionUUID->"f96c5ad7-7250-430a-b5e8-9132dbb22be0",
 CellID->1392205060],
Cell[40055, 1309, 217, 7, 70, "Input",ExpressionUUID->"8fa1678b-2796-4ed6-b964-62b237a456de",
 CellID->2030601230]
}, Open  ]],
Cell[CellGroupData[{
Cell[40309, 1321, 126, 3, 70, "ExampleDelimiter",ExpressionUUID->"3172006d-6f32-4391-b1b8-765d7416ce95",
 CellID->1501491027],
Cell[40438, 1326, 240, 5, 70, "ExampleText",ExpressionUUID->"939d554e-8563-47ca-bfce-76c3b0e4f913",
 CellID->479354892],
Cell[40681, 1333, 218, 7, 70, "Input",ExpressionUUID->"7051aa73-f01d-48ba-bdbb-8db8d39de267",
 CellID->894990216]
}, Open  ]],
Cell[CellGroupData[{
Cell[40936, 1345, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"c8341164-0dbf-40a3-b587-9b5d9dda32c2",
 CellID->215688728],
Cell[41064, 1350, 239, 5, 70, "ExampleText",ExpressionUUID->"dbb88dfd-e65d-427d-a0ed-42d76956d74f",
 CellID->128055360],
Cell[41306, 1357, 173, 5, 70, "Input",ExpressionUUID->"ca712816-83a2-4c66-8b8f-3841278e3bfb",
 CellID->262354246],
Cell[41482, 1364, 168, 5, 70, "Input",ExpressionUUID->"7db817bd-a74d-48d1-a670-227f6e1aa0df",
 CellID->1520002547],
Cell[41653, 1371, 202, 6, 70, "ExampleText",ExpressionUUID->"d56edda7-065a-4faf-893c-fa1902c06cc7",
 CellID->688646404],
Cell[41858, 1379, 150, 5, 70, "Input",ExpressionUUID->"6e26117e-a2fa-454c-943b-2734a1ce96c1",
 CellID->787859925],
Cell[42011, 1386, 176, 4, 70, "ExampleText",ExpressionUUID->"a2b83046-a356-45e8-80ef-7a32f12846e7",
 CellID->319551712],
Cell[42190, 1392, 223, 6, 70, "Input",ExpressionUUID->"2338f5b6-6c1f-4693-9f60-9023557601c9",
 CellID->993691099],
Cell[42416, 1400, 289, 8, 70, "Input",ExpressionUUID->"bad1dd56-9381-4a3c-882f-396f05372f67",
 CellID->1952979937]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[42754, 1414, 197, 6, 70, "ExampleSection",ExpressionUUID->"839e9e9f-5844-44ff-a569-cb40e0c8c29f",
 CellID->1305812373],
Cell[42954, 1422, 133, 4, 70, "ExampleText",ExpressionUUID->"b615a120-ae26-423f-9692-6ec10ac2b6ce",
 CellID->98070909],
Cell[43090, 1428, 846, 23, 70, "Input",ExpressionUUID->"bf461fe9-0976-4369-bc4f-c0a800f574ec",
 CellID->377479142]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[43985, 1457, 31, 0, 70, "SectionFooterSpacer",ExpressionUUID->"5e5bf14a-2b6e-4050-b18e-91d8da3fca1f"],
Cell[44019, 1459, 150, 7, 70, "SeeAlsoSection",ExpressionUUID->"236fab3a-b39b-4936-9f4a-246fe2748a76",
 CellID->1255426704],
Cell[44172, 1468, 947, 28, 70, "SeeAlso",ExpressionUUID->"616d53da-9ac6-42ab-9adb-abce074f6afd",
 CellID->929782353]
}, Open  ]],
Cell[CellGroupData[{
Cell[45156, 1501, 152, 7, 70, "TutorialsSection",ExpressionUUID->"e3be148a-f5b9-44fb-a5eb-eee1cd3fa89d",
 CellID->250839057],
Cell[45311, 1510, 31, 0, 70, "SectionHeaderSpacer",ExpressionUUID->"51604527-3505-4e79-834b-d3dbd29ab0c6"],
Cell[45345, 1512, 180, 5, 70, "Tutorials",ExpressionUUID->"6ed9cf28-c667-40f0-8905-925c99ea19e3",
 CellID->1196281537],
Cell[45528, 1519, 31, 0, 70, "SectionFooterSpacer",ExpressionUUID->"71fc3f18-2057-45e0-bc2a-9280d410b246"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45596, 1524, 156, 7, 70, "MoreAboutSection",ExpressionUUID->"ef417f51-b6ae-4fbc-b014-a31ce46c82bd",
 CellID->38303248],
Cell[45755, 1533, 31, 0, 70, "SectionHeaderSpacer",ExpressionUUID->"8f26262c-9359-4b34-8073-c46db7793412"],
Cell[45789, 1535, 177, 5, 70, "MoreAbout",ExpressionUUID->"ce6de6aa-9e71-41ec-be4f-785d83f94684",
 CellID->1]
}, Open  ]],
Cell[45981, 1543, 23, 0, 70, "FooterCell",ExpressionUUID->"450a7ed9-9565-4a5b-b002-48c2f172bc38"]
}
]
*)

(* End of internal cache information *)
