(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     37510,       1160]
NotebookOptionsPosition[     26324,        928]
NotebookOutlinePosition[     28946,        993]
CellTagsIndexPosition[     28861,        988]
WindowTitle->MultiScaleCakeWaveletStack
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["LIE ANALYSIS SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox[
        InterpretationBox[Cell[TextData[{
          "See Also",
          StyleBox[" \[FilledDownTriangle]", "AnchorBarArrow",
           StripOnInput->False]
         }]],
         TextCell[
          Row[{"See Also", 
            Style[" \[FilledDownTriangle]", "AnchorBarArrow"]}]]],
        StripOnInput->False], {"\<\"CakeWaveletStack\"\>":>
       Documentation`HelpLookup[
       "paclet:LieAnalysis/ref/CakeWaveletStack"], \
"\<\"OrientationScoreTransform\"\>":>Documentation`HelpLookup[
       "paclet:LieAnalysis/ref/OrientationScoreTransform"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"SeeAlso"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox[
        InterpretationBox[Cell[TextData[{
          "Related Guides",
          StyleBox[" \[FilledDownTriangle]", "AnchorBarArrow",
           StripOnInput->False]
         }]],
         TextCell[
          Row[{"Related Guides", 
            Style[" \[FilledDownTriangle]", "AnchorBarArrow"]}]]],
        StripOnInput->False], {"\<\"LieAnalysis Package\"\>":>
       Documentation`HelpLookup["paclet:LieAnalysis/guide/LieAnalysis"]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"MoreAbout"]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox[
        InterpretationBox[Cell[TextData[{
          "URL",
          StyleBox[" \[FilledDownTriangle]", "AnchorBarArrow",
           StripOnInput->False]
         }]],
         TextCell[
          Row[{"URL", 
            Style[" \[FilledDownTriangle]", "AnchorBarArrow"]}]]],
        StripOnInput->
         False], {"\<\"LieAnalysis/ref/MultiScaleCakeWaveletStack\"\>":>
       None, "\<\"Copy Wolfram Documentation Center URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["LieAnalysis/ref/MultiScaleCakeWaveletStack"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web URL\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/language/LieAnalysis/ref/\
MultiScaleCakeWaveletStack.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web URL\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
             If[
              TrueQ[DocumentationBuild`Make`Private`wsmlinkQ$15021], 
              "http://reference.wolfram.com/system-modeler/", 
              "http://reference.wolfram.com/language/"], 
             "LieAnalysis/ref/MultiScaleCakeWaveletStack", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic,
       MenuStyle->"URLMenu"]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[TextData[{
 Cell["MultiScaleCakeWaveletStack", "ObjectName"],
 Cell[BoxData[
  TemplateBox[{8},
   "Spacer1"]]],
 Cell[BoxData[""], "ObjectNameTranslation"]
}], "ObjectNameGrid"],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"MultiScaleCakeWaveletStack", "[", 
       RowBox[{
        StyleBox["orientations", "TI"], ",", " ", 
        StyleBox["size", "TI"], ",", " ", 
        RowBox[{"{", 
         RowBox[{
          StyleBox[
           OverscriptBox[
            StyleBox[
             SubscriptBox[
              StyleBox["\[Rho]", "TI"], "min"], "TI"], "^"], "TI"], ",", " ", 
          
          StyleBox[
           OverscriptBox[
            SubscriptBox["\[Rho]", "max"], "^"], "TI"], ",", " ", 
          StyleBox["n", "TI"]}], "}"}]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]Constructs a multi-scale cake wavelet stack having \
orientations orientations with dimensions ",
     StyleBox["size", "TI"],
     " x ",
     StyleBox["size", "TI"],
     ", and the scales going from ",
     Cell[BoxData[
      OverscriptBox[
       SubscriptBox["\[Rho]", "min"], "^"]], "InlineFormula"],
     " to ",
     Cell[BoxData[
      OverscriptBox[
       SubscriptBox["\[Rho]", "max"], "^"]], "InlineFormula"],
     " using ",
     StyleBox["n", "TI"],
     " scales. "
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[Cell[TextData[{
   Cell[BoxData[
    TemplateBox[{6},
     "Spacer1"]]],
   "Details"
  }], "NotesFrameText"],
  Appearance->{Automatic, None, "Normal", Automatic},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 WholeCellGroupOpener->True,
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->1],

Cell["", "SectionHeaderSpacer"],

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["MultiScaleCakeWaveletStack"]],
    "paclet:LieAnalysis/ref/MultiScaleCakeWaveletStack"},
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 " extends ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["CakeWaveletStack"]],"paclet:LieAnalysis/ref/CakeWaveletStack"},
   
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 " by dividing the wavelets (in the Fourier domain) in log-radial direction."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The radial envelope is defined by:  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"B", 
    RowBox[{"(", 
     OverscriptBox["\[Rho]", "^"], ")"}]}], "=", 
   RowBox[{
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"l", "=", "0"}], 
      SubscriptBox["N", "p"]], 
     RowBox[{
      StyleBox[
       SubsuperscriptBox["B", "l", "k"],
       FontSlant->"Italic"], 
      RowBox[{"(", 
       OverscriptBox["\[Rho]", "^"], ")"}]}]}], " ", ":=", " ", 
    RowBox[{
     UnderoverscriptBox["\[Sum]", 
      RowBox[{"l", "=", "0"}], 
      SubscriptBox["N", "\[Rho]"]], 
     RowBox[{
      SuperscriptBox[
       StyleBox["B",
        FontSlant->"Italic"], "k"], 
      RowBox[{"(", 
       FractionBox[
        RowBox[{"log", 
         RowBox[{"(", 
          RowBox[{
           OverscriptBox["\[Rho]", "^"], " ", 
           SubscriptBox["a", "l"]}], ")"}]}], 
        SubscriptBox["s", "\[Rho]"]], ")"}]}]}]}]}]], "InlineFormula"],
 " with ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["a", "l"], "=", 
   RowBox[{
    SubscriptBox["a", "0"], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"l", " ", 
      SubscriptBox["s", "\[Rho]"]}]]}]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  RowBox[{
   SuperscriptBox[
    StyleBox["B",
     FontSlant->"Italic"], "k"], 
   RowBox[{"(", "x", ")"}]}]], "InlineFormula"],
 " is the ",
 Cell[BoxData[
  SuperscriptBox[
   StyleBox["k",
    FontSlant->"Italic"], "th"]], "InlineFormula"],
 " order ",
 StyleBox["B",
  FontSlant->"Italic"],
 "-spline function, \[Rho] is the radial parameter in the polar coordinates, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["N",
    FontSlant->"Italic"], "\[Rho]"]], "InlineFormula"],
 " is the total number of scales to sample ans ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox["s", "\[Rho]"], " ", ">", " ", "0"}]], "InlineFormula"],
 " is the step size in log-scale."
}], "Notes",
 CellID->765843413],

Cell["The following options can be given:", "Notes",
 CellID->76572729],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], "\"\<Directional\>\"", 
    ButtonBox["False",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/False"], Cell[
    "Whenever the spatial wavelets are single or double headed", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<Design\>\"", "\"\<N\>\"", Cell[
    "Specifies the design constraints for the wavelets", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<SplineOrder\>\"", "3", Cell["\<\
Specifies the order of the B-Spline that is used to create the wavelets\
\>", "TableText"]},
   {Cell["      ", "TableRowIcon"], "\"\<OverlapFactor\>\"", "1", Cell["\<\
Sets how many B-Splines overlap, thereby not narrowing the angular resolution\
\
\>", "TableText"]}
  }]], "3ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, None, {None}}, "RowsIndexed" -> {}},
 GridBoxDividers->{"Rows" -> {{True, True, True, True, True}}}},
 CellID->50244769],

Cell["\<\
Symmetry in the orientation axes is used whenever possible to limit the use \
of resources.\
\>", "Notes",
 CellID->34701960],

Cell["", "SectionFooterSpacer"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{6},
   "Spacer1"]]],
 "Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(14)", "ExampleCount"]
}], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->1],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(2)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1],

Cell[TextData[{
 "Compute a multi scale cake wavelet stack having 12 orientations and 4 \
scales between ",
 Cell[BoxData[
  RowBox[{
   OverscriptBox["\[Rho]", "^"], " ", "=", " ", ".1"}]], "InlineFormula"],
 " and 0.9"
}], "ExampleText",
 CellID->145958011],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Needs", "[", "\"\<LieAnalysis`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MultiScaleCakeWaveletStack", "[", 
  RowBox[{"12", ",", "60", ",", 
   RowBox[{"{", 
    RowBox[{"0.1", ",", "0.9", ",", "4"}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->243172207],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->221584764],

Cell["The shape of the cake wavelets can be modified using options", \
"ExampleText",
 CellID->261641196],

Cell[BoxData[
 RowBox[{"MultiScaleCakeWaveletStack", "[", 
  RowBox[{"12", ",", "60", ",", 
   RowBox[{"{", 
    RowBox[{"0.1", ",", "0.9", ",", "4"}], "}"}], ",", 
   RowBox[{"\"\<SplineOrder\>\"", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->476601188],

Cell["To see the individual wavelets", "ExampleText",
 CellID->14735790],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ImageAdjust", "[", 
    RowBox[{"Image", "[", "#", "]"}], "]"}], "&"}], "/@", 
  RowBox[{"Re", "@", 
   RowBox[{
    RowBox[{"MultiScaleCakeWaveletStack", "[", 
     RowBox[{"12", ",", "60", ",", 
      RowBox[{"{", 
       RowBox[{"0.2", ",", "0.9", ",", "3"}], "}"}]}], "]"}], 
    "\[LeftDoubleBracket]", "\"\<Data\>\"", 
    "\[RightDoubleBracket]"}]}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->168311090]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Scope",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1293636265],

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["MultiScaleCakeWaveletStack"]],
    "paclet:LieAnalysis/ref/MultiScaleCakeWaveletStack"},
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 " returns an ObjScaleCakeWavelet object, which is a child of ObjCakeWavalet \
(which is returned by ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["CakeWaveletStack"]],"paclet:LieAnalysis/ref/CakeWaveletStack"},
   
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 ")"
}], "ExampleText",
 CellID->644924416],

Cell[BoxData[
 RowBox[{"ValidQ", "[", 
  RowBox[{
   RowBox[{"MultiScaleCakeWaveletStack", "[", 
    RowBox[{"12", ",", "60", ",", 
     RowBox[{"{", 
      RowBox[{"0.2", ",", "0.9", ",", "3"}], "}"}]}], "]"}], ",", 
   "ObjCakeWavelet"}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->129614415],

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->374111584]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Generalizations & Extensions",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(2)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1020263627],

Cell[TextData[{
 "You can manipulate the ",
 StyleBox["elements",
  FontSlant->"Italic"],
 " of the ObjScaleCakeWavelet using Affix. Be careful when you do this as ",
 StyleBox["elements",
  FontSlant->"Italic"],
 " might become inconsistent, this can lead to unexpected results."
}], "ExampleText",
 CellID->560092696],

Cell[BoxData[
 RowBox[{"Affix", "[", 
  RowBox[{
   RowBox[{"MultiScaleCakeWaveletStack", "[", 
    RowBox[{"12", ",", "11", ",", 
     RowBox[{"{", 
      RowBox[{".1", ",", ".9", ",", "3"}], "}"}]}], "]"}], ",", 
   RowBox[{"\"\<Symmetry\>\"", "\[Rule]", 
    RowBox[{"2", "\[Pi]"}]}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->785189421],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->998836185],

Cell[TextData[{
 "To create your own orientation wavelet which can be used by the LieAnalysis \
package, you can use the ObjOrientationWavelet format. Do not forget to set \
all ",
 StyleBox["elements ",
  FontSlant->"Italic"],
 "consistently."
}], "ExampleText",
 CellID->1046902088],

Cell[BoxData[
 RowBox[{"ObjOrientationWavelet", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<Data\>\"", "\[Rule]", 
    RowBox[{"RandomReal", "[", 
     RowBox[{"1", ",", 
      RowBox[{"{", 
       RowBox[{"6", ",", "11", ",", "11"}], "}"}]}], "]"}]}], "}"}], 
  "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1158253530]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Options",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(4)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->527810392],

Cell[CellGroupData[{

Cell[TextData[{
 "\"Design\"",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->249054143],

Cell[TextData[{
 "Identical to the ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["CakeWaveletStack"]],"paclet:LieAnalysis/ref/CakeWaveletStack"},
   
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 " the default design used is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["N", "\[Psi]"], "=", 
    RowBox[{
     RowBox[{"\[Integral]", 
      RowBox[{"\[VerticalSeparator]", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["\[Psi]", "^"], "n"], "(", "\[Omega]", ")"}], 
       "\[VerticalSeparator]", " ", 
       RowBox[{"d\[Theta]", "(", "n", ")"}]}]}], " ", "=", " ", "1"}]}], 
   TraditionalForm]]],
 ". Another design that can be used is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["M", "\[Psi]"], "=", 
    RowBox[{
     RowBox[{"\[Integral]", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[VerticalSeparator]", 
          RowBox[{
           SubscriptBox[
            OverscriptBox["\[Psi]", "^"], "n"], "(", "\[Omega]", ")"}], " ", 
          "\[VerticalSeparator]"}], ")"}], "2"], 
       RowBox[{"d\[Theta]", "(", "n", ")"}]}]}], " ", "=", " ", "1"}]}], 
   TraditionalForm]]],
 ". Note that the reconstruction method depends on this setting (See ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["OrientationScoreTransform"]],
    "paclet:LieAnalysis/ref/OrientationScoreTransform"},
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"],
 ")."
}], "ExampleText",
 CellID->127897908],

Cell[BoxData[
 RowBox[{"cws1", " ", "=", " ", 
  RowBox[{"MultiScaleCakeWaveletStack", "[", 
   RowBox[{"12", ",", "23", ",", 
    RowBox[{"{", 
     RowBox[{".1", ",", ".9", ",", "3"}], "}"}], ",", 
    RowBox[{"\"\<Design\>\"", "\[Rule]", "\"\<N\>\""}]}], "]"}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->214102863]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "\"Directional\"",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->545486312],

Cell["\<\
The tails and head of the individual wavelets can be separated by setting \
this option to True. \
\>", "ExampleText",
 CellID->258392697],

Cell[BoxData[
 RowBox[{"MultiScaleCakeWaveletStack", "[", 
  RowBox[{"12", ",", "21", ",", 
   RowBox[{"{", 
    RowBox[{".1", ",", ".7", ",", "3"}], "}"}], ",", 
   RowBox[{"\"\<Directional\>\"", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->200828983]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "\"OverlapFactor\"",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->50765044],

Cell["\<\
The overlap factor is used to sub-sample the fourier spectrum in angular \
direction without narrowing the cake pieces. The resulting Orientation Score \
has smoother transitions between layers in \[Theta]-direction. \
\>", "ExampleText",
 CellID->19611279],

Cell[BoxData[
 RowBox[{"MultiScaleCakeWaveletStack", "[", 
  RowBox[{"12", ",", "21", ",", 
   RowBox[{"{", 
    RowBox[{".1", ",", ".7", ",", "3"}], "}"}], ",", 
   RowBox[{"\"\<OverlapFactor\>\"", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->157055628]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "\"SplineOrder\"",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSubsection", "ExampleSubsection",
 WholeCellGroupOpener->True,
 CellID->247383352],

Cell["\<\
The spline order indicates the order of the used B-spline to construct the \
wavelets. A higher order means more overlap between the wavelets, which means \
that each orientation layer is responsive to a wider range of orientations \
\>", "ExampleText",
 CellID->313761368],

Cell[BoxData[
 RowBox[{"MultiScaleCakeWaveletStack", "[", 
  RowBox[{"12", ",", "23", ",", 
   RowBox[{"{", 
    RowBox[{".1", ",", ".7", ",", "3"}], "}"}], ",", 
   RowBox[{"\"\<SplineOrder\>\"", "\[Rule]", "8"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->354846424]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Applications",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->258228157],

Cell[TextData[{
 "The multi scale cake wavelet stack can be used to compute a multi scale \
orientation score using ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["OrientationScoreTransform"]],
    "paclet:LieAnalysis/ref/OrientationScoreTransform"},
   "RefLink",
   BaseStyle->{"InlineFormula"}]], "InlineFormula"]
}], "ExampleText",
 CellID->165586012],

Cell[BoxData[
 RowBox[{"OrientationScoreTransform", "[", 
  RowBox[{
   RowBox[{"LieAnalysisData", "[", 
    RowBox[{"{", 
     RowBox[{"\"\<image\>\"", ",", "\"\<euler-spirals\>\""}], "}"}], "]"}], 
   ",", 
   RowBox[{"\"\<Type\>\"", "\[Rule]", "\"\<MultiScaleCake\>\""}]}], 
  "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->681163460]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Properties & Relations",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(4)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->2123667759],

Cell[TextData[{
 "To get the data you can request the element ",
 StyleBox["data ",
  FontSlant->"Italic"],
 "which is stored within the object"
}], "ExampleText",
 CellID->385991406],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MultiScaleCakeWaveletStack", "[", 
    RowBox[{"12", ",", " ", "21", ",", 
     RowBox[{"{", 
      RowBox[{".1", ",", ".7", ",", "3"}], "}"}]}], "]"}], 
   "\[LeftDoubleBracket]", "\"\<Data\>\"", "\[RightDoubleBracket]"}], "//", 
  "Dimensions"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->91228733],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->246280449],

Cell["\<\
Elements that can be stored in the object can be found through the property \
defaults\
\>", "ExampleText",
 CellID->6154018],

Cell[BoxData[
 RowBox[{"Framed", "/@", 
  RowBox[{"Keys", "@", 
   RowBox[{
    RowBox[{"MultiScaleCakeWaveletStack", "[", 
     RowBox[{"12", ",", "21", ",", 
      RowBox[{"{", 
       RowBox[{".1", ",", ".7", ",", "3"}], "}"}]}], "]"}], "[", 
    "\"\<Defaults\>\"", "]"}]}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->246023198]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->102213024],

Cell["\<\
The output object contains many properties which are computed when requested\
\>", "ExampleText",
 CellID->48714594],

Cell[BoxData[
 RowBox[{"Framed", "/@", 
  RowBox[{
   RowBox[{"MultiScaleCakeWaveletStack", "[", 
    RowBox[{"12", ",", "11", ",", 
     RowBox[{"{", 
      RowBox[{".1", ",", ".7", ",", "3"}], "}"}]}], "]"}], "[", 
   "\"\<Properties\>\"", "]"}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->139238934]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->148443734],

Cell[TextData[{
 "An important element is ",
 StyleBox["Data ",
  FontSlant->"Italic"],
 "and property ",
 StyleBox["FullData ",
  FontSlant->"Italic"],
 "which contain the wavelets. Internally symmetry is used whenever possible. \
The ",
 StyleBox["Data ",
  FontSlant->"Italic"],
 "element uses symmetry if available, if you want the full orientation axes \
you can request the property ",
 StyleBox["FullData",
  FontSlant->"Italic"]
}], "ExampleText",
 CellID->727163462],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MultiScaleCakeWaveletStack", "[", 
    RowBox[{"12", ",", " ", "21", ",", 
     RowBox[{"{", 
      RowBox[{".1", ",", ".7", ",", "3"}], "}"}]}], "]"}], 
   "\[LeftDoubleBracket]", "\"\<Data\>\"", "\[RightDoubleBracket]"}], "//", 
  "Dimensions"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->209564665],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MultiScaleCakeWaveletStack", "[", 
    RowBox[{"12", ",", " ", "21", ",", 
     RowBox[{"{", 
      RowBox[{".1", ",", ".7", ",", "3"}], "}"}]}], "]"}], "[", 
   "\"\<FullData\>\"", "]"}], "//", "Dimensions"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->49124269],

Cell[TextData[{
 "Note that it is possible that the element ",
 StyleBox["Data",
  FontSlant->"Italic"],
 " and property ",
 StyleBox["FullData ",
  FontSlant->"Italic"],
 "are the same when no symmetry can be used"
}], "ExampleText",
 CellID->422189839],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MultiScaleCakeWaveletStack", "[", 
    RowBox[{"13", ",", " ", "21", ",", 
     RowBox[{"{", 
      RowBox[{".1", ",", ".7", ",", "3"}], "}"}]}], "]"}], "[", 
   "\"\<FullData\>\"", "]"}], "===", 
  RowBox[{
   RowBox[{"MultiScaleCakeWaveletStack", "[", 
    RowBox[{"13", ",", " ", "21", ",", 
     RowBox[{"{", 
      RowBox[{".1", ",", ".7", ",", "3"}], "}"}]}], "]"}], 
   "\[LeftDoubleBracket]", "\"\<Data\>\"", 
   "\[RightDoubleBracket]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->281085920]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["", "SectionFooterSpacer"],

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{6},
   "Spacer1"]]],
 "See Also"
}], "SeeAlsoSection",
 WholeCellGroupOpener->True,
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["CakeWaveletStack"]],"paclet:LieAnalysis/ref/CakeWaveletStack"},
   
   "RefLink",
   BaseStyle->{"InlineFormula", FontFamily -> "Verdana"}]], "InlineFormula"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[BoxData[
  TemplateBox[{Cell[
     TextData["OrientationScoreTransform"]],
    "paclet:LieAnalysis/ref/OrientationScoreTransform"},
   "RefLink",
   BaseStyle->{"InlineFormula", FontFamily -> "Verdana"}]], "InlineFormula"]
}], "SeeAlso",
 CellID->929782353]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 Cell[BoxData[
  TemplateBox[{6},
   "Spacer1"]]],
 "Related Guides"
}], "MoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->38303248],

Cell["", "SectionHeaderSpacer"],

Cell[BoxData[
 TemplateBox[{Cell[
    TextData["LieAnalysis Package"]],"paclet:LieAnalysis/guide/LieAnalysis"},
  "RefLink",
  BaseStyle->"MoreAbout"]], "MoreAbout",
 CellID->1]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"MultiScaleCakeWaveletStack",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "LieAnalysis Package", ButtonData -> 
         "paclet:LieAnalysis/guide/LieAnalysis", 
         BaseStyle -> {"Link", "DockedLinkTrail"}]}]}}, ColumnAlignments -> 
    Left], "HasOptions" -> False, "ExampleCounter" -> 1, "NeedPlatMsgIn" -> 
  None, "RootCaptions" -> "", 
  "Metadata" -> {
   "built" -> "{2017, 10, 26, 15, 6, 58.8412251}", 
    "history" -> {"0.4", "1.0", "", ""}, "context" -> "LieAnalysis`", 
    "keywords" -> {
     "Cake wavelets", "Multi Scale", "Multi Scale Cake wavelets", 
      "Orientation Score", "OrientationScoreTransform"}, 
    "specialkeywords" -> {}, "tutorialcollectionlinks" -> {}, "index" -> True,
     "label" -> "Lie Analysis Symbol", "language" -> "en", "paclet" -> 
    "LieAnalysis", "status" -> "None", "summary" -> 
    "MultiScaleCakeWaveletStack[orientations, size, { \
OverscriptBox[StyleBox[RowBox[{ StyleBox[\"\\[Rho]\", \"TI\"], \"min\"}], \
\"TI\"], ^], OverscriptBox[\\[Rho]max, ^], n}] Constructs a multi-scale cake \
wavelet stack having orientations orientations with dimensions size x size, \
and the scales going from OverscriptBox[\\[Rho]min, ^] to \
OverscriptBox[\\[Rho]max, ^] using n scales. ", "synonyms" -> {}, 
    "tabletags" -> {}, "title" -> "MultiScaleCakeWaveletStack", 
    "titlemodifier" -> "", "windowtitle" -> "MultiScaleCakeWaveletStack", 
    "type" -> "Symbol", "uri" -> 
    "LieAnalysis/ref/MultiScaleCakeWaveletStack"}},
CellContext->"Global`",
FrontEndVersion->"11.2 for Microsoft Windows (64-bit) (September 10, 2017)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> 
  "11.2 for Microsoft Windows (64-bit) (September 10, 2017)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[10361, 320, 260, 10, 70, "PrimaryExamplesSection",ExpressionUUID->"bd01611c-3d79-48d0-820a-682c3df5997d",
   CellTags->"PrimaryExamplesSection",
   CellID->1]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 28670, 981}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[598, 21, 3818, 95, 70, "AnchorBarGrid",ExpressionUUID->"3721d3fe-979d-4e5b-bce6-8c88304ab102",
 CellID->1],
Cell[4419, 118, 183, 6, 70, "ObjectNameGrid",ExpressionUUID->"7a196689-2d41-404b-8438-25fc4c7b3b75"],
Cell[4605, 126, 1319, 41, 70, "Usage",ExpressionUUID->"09ebd5cd-7835-46f0-a438-95eaab97bc36",
 CellID->982511436],
Cell[CellGroupData[{
Cell[5949, 171, 621, 19, 70, "NotesSection",ExpressionUUID->"cccbc326-13d2-42b9-a4d9-22dc5bcf1bc1",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->1],
Cell[6573, 192, 31, 0, 70, "SectionHeaderSpacer",ExpressionUUID->"0d88fe24-c05d-4b4b-b47c-24f067b8e780"],
Cell[6607, 194, 534, 16, 70, "Notes",ExpressionUUID->"da33184f-f31f-4ff8-8753-19687a33073a",
 CellID->1067943069],
Cell[7144, 212, 1926, 69, 70, "Notes",ExpressionUUID->"9ae641f7-fa1c-4f8f-9ce8-2194d8b29228",
 CellID->765843413],
Cell[9073, 283, 71, 1, 70, "Notes",ExpressionUUID->"f0811314-2263-46cf-a6b3-fa093311044f",
 CellID->76572729],
Cell[9147, 286, 1005, 21, 70, "3ColumnTableMod",ExpressionUUID->"84ac56a4-7113-44c3-ae35-57a1e48ddde9",
 CellID->50244769],
Cell[10155, 309, 135, 4, 70, "Notes",ExpressionUUID->"b1c51927-d1fe-403d-987a-d5dac8480fbb",
 CellID->34701960],
Cell[10293, 315, 31, 0, 70, "SectionFooterSpacer",ExpressionUUID->"20670868-6225-4db2-bef1-2efe8162ce91"]
}, Closed]],
Cell[CellGroupData[{
Cell[10361, 320, 260, 10, 70, "PrimaryExamplesSection",ExpressionUUID->"bd01611c-3d79-48d0-820a-682c3df5997d",
 CellTags->"PrimaryExamplesSection",
 CellID->1],
Cell[CellGroupData[{
Cell[10646, 334, 187, 6, 70, "ExampleSection",ExpressionUUID->"41b02215-e928-47c1-b8c7-dade9f40b708",
 CellID->1],
Cell[10836, 342, 259, 8, 70, "ExampleText",ExpressionUUID->"1c02a497-3554-47d6-b9fc-5ba7137b13b5",
 CellID->145958011],
Cell[11098, 352, 321, 9, 70, "Input",ExpressionUUID->"2f6c08fc-7b0f-4c3c-872d-7922be29e392",
 CellID->243172207],
Cell[CellGroupData[{
Cell[11444, 365, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"8c796d34-57c6-4c37-9d2d-becf9bcf7b65",
 CellID->221584764],
Cell[11572, 370, 105, 2, 70, "ExampleText",ExpressionUUID->"0b28ca7b-e052-4859-b61a-5124a6809adf",
 CellID->261641196],
Cell[11680, 374, 278, 7, 70, "Input",ExpressionUUID->"7ff075f7-6344-476d-b83d-4052f1cb41ab",
 CellID->476601188],
Cell[11961, 383, 72, 1, 70, "ExampleText",ExpressionUUID->"807db083-a926-4701-b5a0-18c6bce2715e",
 CellID->14735790],
Cell[12036, 386, 459, 14, 70, "Input",ExpressionUUID->"490b6024-1495-47d3-9798-1bb2ac4d11da",
 CellID->168311090]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[12544, 406, 187, 6, 70, "ExampleSection",ExpressionUUID->"3315f4ae-c18d-4cea-9922-5db0f8b2c0e2",
 CellID->1293636265],
Cell[12734, 414, 556, 17, 70, "ExampleText",ExpressionUUID->"2480e440-bbf2-4f1d-a322-74427d33877f",
 CellID->644924416],
Cell[13293, 433, 300, 9, 70, "Input",ExpressionUUID->"20049676-37ea-4543-b68b-1b871978c42b",
 CellID->129614415],
Cell[13596, 444, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"a0b26e85-a3e4-4674-acf4-3b000cbf2587",
 CellID->374111584]
}, Closed]],
Cell[CellGroupData[{
Cell[13758, 452, 210, 6, 70, "ExampleSection",ExpressionUUID->"89f70047-5798-4996-9436-28cf46444c95",
 CellID->1020263627],
Cell[13971, 460, 319, 9, 70, "ExampleText",ExpressionUUID->"8ade81bb-8f50-4ec0-be26-4a4701251601",
 CellID->560092696],
Cell[14293, 471, 349, 10, 70, "Input",ExpressionUUID->"d92624b0-a32e-4c02-a7eb-a50d6f74b5b2",
 CellID->785189421],
Cell[CellGroupData[{
Cell[14667, 485, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"5452ec5b-7ad3-46c4-ae5c-6c6e5b0eddcd",
 CellID->998836185],
Cell[14795, 490, 284, 8, 70, "ExampleText",ExpressionUUID->"f844ab48-b91e-431d-8647-e0eabde8f7ab",
 CellID->1046902088],
Cell[15082, 500, 317, 10, 70, "Input",ExpressionUUID->"ce1194cc-d72f-47b9-810a-3f328bfccbeb",
 CellID->1158253530]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[15448, 516, 188, 6, 70, "ExampleSection",ExpressionUUID->"604c86fc-2738-4ab1-8db7-1f15fd9a6f2b",
 CellID->527810392],
Cell[CellGroupData[{
Cell[15661, 526, 197, 6, 70, "ExampleSubsection",ExpressionUUID->"feaf408d-97e6-4e8e-ac3e-222fb56b18e8",
 CellID->249054143],
Cell[15861, 534, 1510, 48, 70, "ExampleText",ExpressionUUID->"db88fda8-a843-4efe-a41b-987a909b7a6b",
 CellID->127897908],
Cell[17374, 584, 319, 8, 70, "Input",ExpressionUUID->"561dff3b-b35d-4f0e-8ad7-c44915ecd10e",
 CellID->214102863]
}, Closed]],
Cell[CellGroupData[{
Cell[17730, 597, 202, 6, 70, "ExampleSubsection",ExpressionUUID->"30b25be2-1c13-4ee8-8f87-596cfd1cea13",
 CellID->545486312],
Cell[17935, 605, 148, 4, 70, "ExampleText",ExpressionUUID->"80322c4f-d557-4b62-a5eb-4f6a798bc70c",
 CellID->258392697],
Cell[18086, 611, 279, 7, 70, "Input",ExpressionUUID->"9f99f91d-a9cd-4104-ba07-5b7cf7b28a91",
 CellID->200828983]
}, Closed]],
Cell[CellGroupData[{
Cell[18402, 623, 203, 6, 70, "ExampleSubsection",ExpressionUUID->"965ad642-a48a-45ef-9510-0984fecd3582",
 CellID->50765044],
Cell[18608, 631, 267, 5, 70, "ExampleText",ExpressionUUID->"c2f0ace6-b23a-4cad-8a3b-f3b50093de3f",
 CellID->19611279],
Cell[18878, 638, 278, 7, 70, "Input",ExpressionUUID->"b40626ea-ec98-4f2c-a7d3-e3096037dd3b",
 CellID->157055628]
}, Closed]],
Cell[CellGroupData[{
Cell[19193, 650, 202, 6, 70, "ExampleSubsection",ExpressionUUID->"38a3c94c-10b6-45f0-a187-d12c07672441",
 CellID->247383352],
Cell[19398, 658, 283, 5, 70, "ExampleText",ExpressionUUID->"0de56718-64a6-4d37-b798-a8afee388352",
 CellID->313761368],
Cell[19684, 665, 276, 7, 70, "Input",ExpressionUUID->"9d3507ad-96b9-4803-842c-cf7454786362",
 CellID->354846424]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[20009, 678, 193, 6, 70, "ExampleSection",ExpressionUUID->"38ff8b66-7ae8-4a14-a08b-16159c7a1b51",
 CellID->258228157],
Cell[20205, 686, 361, 10, 70, "ExampleText",ExpressionUUID->"f6781ad5-dde1-4bdd-a162-3ac6689b9bda",
 CellID->165586012],
Cell[20569, 698, 339, 10, 70, "Input",ExpressionUUID->"7da12399-76b4-4781-bc64-a85f36068227",
 CellID->681163460]
}, Closed]],
Cell[CellGroupData[{
Cell[20945, 713, 204, 6, 70, "ExampleSection",ExpressionUUID->"306dd3ee-507d-46f1-9302-b7d1265f5d83",
 CellID->2123667759],
Cell[21152, 721, 183, 6, 70, "ExampleText",ExpressionUUID->"f7b30514-9ebc-44dd-88e8-c83e3aea5e91",
 CellID->385991406],
Cell[21338, 729, 347, 10, 70, "Input",ExpressionUUID->"41e8d06d-1e20-4b10-8cde-b8d4974ddb05",
 CellID->91228733],
Cell[CellGroupData[{
Cell[21710, 743, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"3804f195-b9e0-46a3-8722-17e1aa084d17",
 CellID->246280449],
Cell[21838, 748, 135, 4, 70, "ExampleText",ExpressionUUID->"d0f5dcd6-0b4e-49bc-b6c9-355a674ecbfa",
 CellID->6154018],
Cell[21976, 754, 333, 10, 70, "Input",ExpressionUUID->"e474322f-aa96-45de-9140-5159bb0e7764",
 CellID->246023198]
}, Open  ]],
Cell[CellGroupData[{
Cell[22346, 769, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"afc0bab1-44bb-4d75-8507-1d88e3c1299d",
 CellID->102213024],
Cell[22474, 774, 126, 3, 70, "ExampleText",ExpressionUUID->"3cdded1e-1dc1-4916-a822-d9e968ba0ea8",
 CellID->48714594],
Cell[22603, 779, 303, 9, 70, "Input",ExpressionUUID->"11805904-e7e4-4506-91ff-eec45aa1d1c7",
 CellID->139238934]
}, Open  ]],
Cell[CellGroupData[{
Cell[22943, 793, 125, 3, 70, "ExampleDelimiter",ExpressionUUID->"8eaf13a9-4d9e-4c36-ad2c-32c04464617b",
 CellID->148443734],
Cell[23071, 798, 475, 16, 70, "ExampleText",ExpressionUUID->"48be74d4-4c8b-4a76-8b2c-1e22630b5cd6",
 CellID->727163462],
Cell[23549, 816, 348, 10, 70, "Input",ExpressionUUID->"603fb799-91a4-4153-a401-75d547531584",
 CellID->209564665],
Cell[23900, 828, 309, 9, 70, "Input",ExpressionUUID->"0cd2098c-c052-4b33-a139-649afed35896",
 CellID->49124269],
Cell[24212, 839, 254, 9, 70, "ExampleText",ExpressionUUID->"fc19580e-6671-461f-a39d-aadf4e3af4cf",
 CellID->422189839],
Cell[24469, 850, 549, 16, 70, "Input",ExpressionUUID->"aff7872e-cfac-4747-8cc0-3b90ea6c60af",
 CellID->281085920]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[25079, 873, 31, 0, 70, "SectionFooterSpacer",ExpressionUUID->"98e91e6b-e00e-48ea-b403-d45129d87e84"],
Cell[25113, 875, 150, 7, 70, "SeeAlsoSection",ExpressionUUID->"c7f5fc6b-7bf2-418f-972d-4079e3244e51",
 CellID->1255426704],
Cell[25266, 884, 609, 18, 70, "SeeAlso",ExpressionUUID->"2348b287-3f91-43f0-901c-7e64d4994421",
 CellID->929782353]
}, Open  ]],
Cell[CellGroupData[{
Cell[25912, 907, 156, 7, 70, "MoreAboutSection",ExpressionUUID->"79fb36a5-58a4-453b-a0a5-1531a98ee6be",
 CellID->38303248],
Cell[26071, 916, 31, 0, 70, "SectionHeaderSpacer",ExpressionUUID->"d5bf79cc-d396-44f7-ad2d-d2a2d351d8aa"],
Cell[26105, 918, 177, 5, 70, "MoreAbout",ExpressionUUID->"11f8ee45-4e47-4b59-8b6f-e47312f3ca06",
 CellID->1]
}, Open  ]],
Cell[26297, 926, 23, 0, 70, "FooterCell",ExpressionUUID->"fb1295cc-2ef8-4446-a5a6-bbcde6457eb2"]
}
]
*)

(* End of internal cache information *)
